﻿/* 
 * Autor: Jesús Eduardo Méndez Rosales
 * 
 * Descripción: Aplicación que utiliza el puerto serial para realizar comunicación con Avanxe 7
 *              La aplicación opera a 3 Megabaudios y la transmisión y recepción se hace de manera asíncrona.
 *              Utiliza una imagen en formato PNG para obtener el color en formato RGB (con rango de 0 a 255) de cada pixel mediante el posicionamiento del cursor, 
 *              dicha información se transmite por RS232.
 *              Para la recepción de datos se utiliza un hilo que se ejecuta en segundo plano.
 *              
 *              Combo Box:
 *                  comboBoxPuertosCOM - Almacena el nombre de los puertos COM disponibles.
 *              
 *              Botones:
 *                  btnRefresh - Vuelve a hacer la búsqueda de puertos COM para mostrarlos en comboBoxPuertosCOM.
 *                  btnAbrir   - Configura y abre el puerto COM seleccionado.
 *              
 *              Etiquetas:
 *                  lblStatus  - Se muestra "Puerto abierto" si se logró abrir el puerto COM o "No se pudo abrir el puerto" por cualquier error.
 *                  lblByte    - Muestra el dato que se recibe.
 *                  
 *              Picture Box:
 *                  pictureBoxImagen   - Espacio donde se muestra la imagen "color.png" y se obtendrá la información de los colores en formato RGB.
 *                  pictureBoxColorSel - Espacio donde se muestra el color seleccionado  por el cursor.
 * 
 * 
 * */


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;
using System.Threading;

namespace Main
{
    public partial class Form1 : Form
    {
        //Variables que almacenan la información de cada color.
        int R = 0; //Rojo
        int G = 0; //Verde
        int B = 0; //Azul

        /*Boleano que permite la transmisión de datos. Se utiliza para evitar que la aplicación se crashee en caso de que no se haya abierto
        el puerto COM y el cursor se posicione sobre la imagen.*/
        bool ok = false; 

        //Objeto SerialPort.
        public SerialPort sp = new SerialPort();

        //Objetos para crear un hilo (Thread).
        Thread t;
        Hilo hilo;

        public Form1()
        {
            InitializeComponent();

            //Cargamos la imagen.
            pictureBoxImagen.Image = Image.FromFile(System.AppDomain.CurrentDomain.BaseDirectory+"color.png");

            //Obtenemos nombres de puertos COM.
            string[] nombres = SerialPort.GetPortNames();

            //Instrucción que permite modificar elementos de la interfas dentro de un hilo.
            CheckForIllegalCrossThreadCalls = false;

            //Creando objetos para el hilo
            hilo = new Hilo(this);
            t = new Thread(hilo.Metodo);

            //Llenamos a comboBoxPuertosCOM con los nombres de los puertos.
            for (int i = 0; i < nombres.Length; i++)
            {
                comboBoxPuertosCOM.Items.Add(nombres[i]);
            }

        }

        //Acción del boton Refresh para volver a buscar los puertos COM.
        private void button1_Click(object sender, EventArgs e)
        {
            comboBoxPuertosCOM.Items.Clear();


            string[] nombres = SerialPort.GetPortNames();

            for (int i = 0; i < nombres.Length; i++)
            {
                comboBoxPuertosCOM.Items.Add(nombres[i]);
            }

        }

        //Abriendo puerto COM.
        private void button2_Click(object sender, EventArgs e)
        {
            try
            {
                //Configurando puerto
                sp.BaudRate = 3000000;
                sp.DataBits = 8;
                sp.Handshake = Handshake.None;
                sp.Parity = Parity.None;
                sp.PortName = comboBoxPuertosCOM.SelectedItem.ToString();
                sp.Open();
                lblStatus.Text = "¡Puerto Abierto!";
                //Activa al boleano "ok".
                ok = true;
                //Ejecuta el hilo para la recepción de datos.
                t.Start();
                //Envia un dato para que inicialice el contador en Avanxe 7.
                byte[] a = { 0, 0, 0 };
                sp.Write(a, 0, 3);
            }
            catch
            {
                //Si se produjo un error al abrir el puerto se indica con la etiqueta.
                lblStatus.Text = "Error al abrir el puerto";
            }
        }

        //Método que obtiene la información de cada pixel de la imagen cuando el cursor se posiciona encima.
        private void pictureBox1_MouseMove(object sender, MouseEventArgs e)
        {
            if (ok)
            {
                //Obtenemos coordenadas de la imagen
                int x = e.X;
                int y = e.Y;
                

                // Create a Bitmap object from an image file.
                Bitmap myBitmap = new Bitmap(System.AppDomain.CurrentDomain.BaseDirectory + "color.png");

                // Get the color of a pixel within myBitmap.
                Color pixelColor = myBitmap.GetPixel(x, y);
               
                //Obtenemos la información de cada color en un rango de 0-255 (0x00-0xFF).
                R = pixelColor.R;
                G = pixelColor.G;
                B = pixelColor.B;

                //Creamos el array de datos y transmitimos los datos.
                byte[] x2 = { (byte)R, (byte)G, (byte)B };
                sp.Write(x2, 0, 3);

                //Mandamos el color seleccionado en pictureBoxColorSel.
                pictureBoxColorSel.BackColor = pixelColor;
            }
        }
    }
}
